package example_person_accounts_array;

public class Person {
	private String name;
	
	private BasicAccount[] accounts = new BasicAccount[10];
	private int numAccounts = 0;

	public Person(String name) {
		this.name = name;
	}
	
	public String getName() {
		return name;
	}

	public int getNumAccounts() {
		return numAccounts;
	}
	
	public void addAccount(BasicAccount a) {
		if(numAccounts<accounts.length) {
			accounts[numAccounts++] = a;
		}
	}
	
	public void addAccounts(BasicAccount[] accounts) {
		for(BasicAccount a : accounts) {
			addAccount(a);
		}
	}
	
	public double getTotalGoldAccounts() {
		double sum = 0.0;
		for(int i=0; i<numAccounts; i++) {
			BasicAccount a = accounts[i];
			//if((a instanceof GoldAccount) && !(a instanceof PlatinumAccount)) {
			if(a instanceof GoldAccount) {
				sum += a.getBalance();
			}
		}
		return sum;
	}

	
	public double getSmallestInterestRate() {
		double smallestIntRate = Double.MAX_VALUE;
		for(int i=0; i<numAccounts; i++) {
			BasicAccount a = accounts[i];
			if(a instanceof GoldAccount) {
				GoldAccount ga = (GoldAccount)a;
				if(ga.getInterestRate()<smallestIntRate) {
					smallestIntRate = ga.getInterestRate();
				}
			}
		}
		return smallestIntRate;
	}
	
	public GoldAccount getGoldAccountWithSmallestInterestRate() {
		GoldAccount gaSmallest = null;
		double smallestIntRate = Double.MAX_VALUE;
		for(int i=0; i<numAccounts; i++) {
			BasicAccount a = accounts[i];
			if(a instanceof GoldAccount) {
				GoldAccount ga = (GoldAccount)a;
				if(ga.getInterestRate()<smallestIntRate) {
					smallestIntRate = ga.getInterestRate();
					gaSmallest = ga;
				}
			}
		}
		return gaSmallest;
	}

	public GoldAccount[] getGoldAccounts() {
		GoldAccount[] gAcnts = new GoldAccount[getNumGoldAccounts()];
		int j=0;
		for(int i=0; i<numAccounts; i++) {
			BasicAccount a = accounts[i];
			if(a instanceof GoldAccount) {
				gAcnts[j++] = (GoldAccount)a;
			}
		}
		return gAcnts;
	}
	
	// Helper method to return the number of GoldAccounts
	private int getNumGoldAccounts() {
		int count=0;
		for(int i=0; i<numAccounts; i++) {
			if(accounts[i] instanceof GoldAccount) {
				count++;
			}
		}
		return count;
	}

	public void applyInterest() {
		for(int i=0; i<numAccounts; i++)
			accounts[i].applyInterest();
	}
	
	public BasicAccount getAccount(int i) {
		if(i>=0 && i<numAccounts) {
			return accounts[i];
		}
		return null;
	}
	
	public double getTotalBalance() {
		double sum=0.0;
		for(int i=0; i<numAccounts; i++) {
			sum += accounts[i].getBalance();
		}
		return sum;
	}

	public BasicAccount removeAccount(int i) {
		if(i<0 || i>=numAccounts) {
			return null;
		}
		BasicAccount retAccount = accounts[i];
		
		for(int j=i+1; j<numAccounts; j++) {
			accounts[j-1] = accounts[j];
		}
		numAccounts--;
		return retAccount;
		
	}
	
//	// Incorrect, uses length of array, some elements
//	// may not have accounts. Used for notes
//	public double getTotalBalance2() {
//		double sum=0.0;
//		for(int i=0; i<accounts.length; i++) {
//			sum += accounts[i].getBalance();
//		}
//		return sum;
//	}
//	
//	// Incorrect, uses entire array, some elements
//	// may not have accounts. Used for notes
//	public double getTotalBalance3() {
//		double sum=0.0;
//		for(BasicAccount a : accounts) {
//			sum += a.getBalance();
//		}
//		return sum;
//	}

	@Override
	public String toString() {
		String msg = String.format("Name: %s, Tot Bal=$%,.2f, Num Acnts=%d\n", 
				name, getTotalBalance(), getNumAccounts());
		for(int i=0; i<numAccounts; i++) {
			String acnt = String.format("%d - %s\n", i+1, accounts[i]);
			msg += acnt;
		}
		return msg;
	}
}
