package example_person_accounts_arraylist;

import java.util.ArrayList;

public class Person {
	
	private String name;
	
	private ArrayList<BasicAccount> accounts = new ArrayList<>();

	public Person(String name) {
		this.name = name;
	}
	
	public String getName() {
		return name;
	}
	
	public int getNumAccounts() {
		return accounts.size();
	}
	
	// Add account if doesn't already exist
	public boolean addAccount(BasicAccount a) {
		if(accounts.contains(a)) {
			return false;
		}
		accounts.add(a);
		return true;
	}
	
	public BasicAccount getAccount(int i) {
		if(i>=0 && i<accounts.size()) {
			return accounts.get(i);
		}
		return null;
	}

	// Get an account based on accountNumber
	public BasicAccount getAccount(String accountNumber) {
		// Create dummy account
		BasicAccount acntKey = new BasicAccount(accountNumber);
		// Find location of dummy account in list
		int pos = accounts.indexOf(acntKey);
		// If the location is 0 or greater, then it was found
		if(pos>=0) {
			// Return the real (matching) account
			return accounts.get(pos);
		}
		// If the location was not found, return null.
		return null;
	}

	// Returns accounts whose first characters match partialNum
	public ArrayList<BasicAccount> getAccountsWithNumber(String partialNum) {
		ArrayList<BasicAccount> acntMatches = new ArrayList<>();
		int len = partialNum.length();
		for(BasicAccount a : accounts) {
			if(a.getAccountNumber().substring(0,len).equals(partialNum)) {
				acntMatches.add(a);
			}
		}
		return acntMatches;
	}
	
	public BasicAccount removeAccount(int i) {
		if(i>=0 && i<accounts.size()) {
			BasicAccount retAccount = accounts.get(i);
			accounts.remove(i);
			return retAccount;
		}
		return null;
	}
	
	public BasicAccount removeAccountAlternate(int i) {
		if(i>=0 && i<accounts.size()) {
			return accounts.remove(i);
		}
		return null;
	}

	// Remove based on account number
	public BasicAccount removeAccount(String accountNumber) {
		BasicAccount acntKey = new BasicAccount(accountNumber);
		int pos = accounts.indexOf(acntKey);
		if(pos>=0) {
			return accounts.remove(pos);  // uses index to remove
		}
		return null;
	}
	
	//Remove based on account number
	public BasicAccount removeAccountAlternate(String accountNumber) {
		BasicAccount acntKey = new BasicAccount(accountNumber);
		int pos = accounts.indexOf(acntKey);
		if(pos>=0) {
			BasicAccount retAccount = accounts.get(pos);
			accounts.remove(acntKey); // uses Object to remove
			return retAccount;
		}
		return null;
	}

	// Removed this method, just b/c probably don't need it
//	public void addAccounts(ArrayList<BasicAccount> accountsNew) {
//		accounts.addAll(accountsNew);
//	}


	
	public double getTotalGoldAccounts() {
		double sum = 0.0;
		for(BasicAccount a : accounts) {
			if(a instanceof GoldAccount) {
				sum += a.getBalance();
			}
		}
		return sum;
	}

	
	public double getSmallestInterestRate() {
		double smallestIntRate = Double.MAX_VALUE;
		for(BasicAccount a : accounts) {
			if(a instanceof GoldAccount) {
				GoldAccount ga = (GoldAccount)a;
				if(ga.getInterestRate()<smallestIntRate) {
					smallestIntRate = ga.getInterestRate();
				}
			}
		}
		return smallestIntRate;
	}
	
	public ArrayList<GoldAccount> getGoldAccounts() {
		ArrayList<GoldAccount> gAcnts = new ArrayList<>();
		for(BasicAccount a : accounts) {
			if(a instanceof GoldAccount) {
				gAcnts.add((GoldAccount)a);
			}
		}
		return gAcnts;
	}
	
	public void applyInterest() {
		for(BasicAccount a : accounts) {
			a.applyInterest();
		}
	}
	
	public void applyInterestAlternate() {
		for(int i=0; i<accounts.size(); i++) {
			accounts.get(i).applyInterest();
		}
	}

	// Check to see if an account exists based on accountNumber
	public boolean hasAccount(String accountNumber) {
		BasicAccount acntKey = new BasicAccount(accountNumber);
		int pos = accounts.indexOf(acntKey);
		if(pos>=0) {
			return true;
		}
		return false;
	}
	
	public double getTotalBalance() {
		double sum=0.0;
		for(BasicAccount a : accounts) {
			sum += a.getBalance();
		}
		return sum;
	}

	@Override
	public String toString() {
		String msg = String.format("Name: %s, Tot Bal=$%,.2f, Num Acnts=%d\n", 
				name, getTotalBalance(), getNumAccounts());
		int i=1;
		for(BasicAccount a : accounts) {
			String acnt = String.format("%d - %s\n", i++, a);
			msg += acnt;
		}
		return msg;
	}
}
