package example_person_accounts_arraylist;

import java.util.ArrayList;

public class PersonTest {

	public static void main(String[] args) {
		testAddAccount();
		testAddAccount_Duplicate();
		testGetAccount_Success();
		testGetAccount_Fail();
		getAccountWithNumber_Success();
		getAccountWithNumber_Fail();		
		getAccountsWithNumber_Success();
		getAccountsWithNumber_Fail();
		removeAccount_Success();
		removeAccount_Fail();
		removeAccountWithNumber_Success();
		removeAccountWithNumber_Fail();	
		hasAccount_Success();		
		hasAccount_Fail();		
		testApplyInterest();
		testGetTotalBalance();	
		testGetTotalGoldAccounts();	
		testGetSmallestInterestRate();
		testGetGoldAccounts();		
	}

	private static void testAddAccount() {
		System.out.println("-->testAddAccount()");
		Person p = new Person("Nate");
		BasicAccount ba = new BasicAccount("123", 1000.0);
		GoldAccount ga1 = new GoldAccount("111", 500.0, 0.1);
		p.addAccount(ba);
		p.addAccount(ga1);
		System.out.println("added 2 accounts");
		System.out.println("Num Accounts:" + p.getNumAccounts());
	}
	
	private static void testAddAccount_Duplicate() {
		System.out.println("\n-->testAddAccount_Duplicate()");
		Person p = new Person("Nate");
		BasicAccount ba = new BasicAccount("123", 1000.0);
		GoldAccount ga1 = new GoldAccount("123", 500.0, 0.1);

		System.out.println("Tried to add these accounts");
		boolean isAdded = p.addAccount(ba);
		System.out.println("isAdded=" + isAdded + ", " + ba);

		isAdded = p.addAccount(ga1);
		System.out.println("isAdded=" + isAdded + ", " + ga1);
		System.out.println("Num Accounts:" + p.getNumAccounts());
	}

	private static void testGetAccount_Success() {
		System.out.println("\n-->testGetAccount_Success()");
		Person p = new Person("Nate");
		BasicAccount ba = new BasicAccount("123", 1000.0);
		GoldAccount ga1 = new GoldAccount("111", 500.0, 0.1);
		p.addAccount(ba);
		p.addAccount(ga1);
		BasicAccount a = p.getAccount(1);
		System.out.println("Account trying to retrieve:" + ga1);
		System.out.println("           p.getAccount(1):" + a);
	}

	private static void testGetAccount_Fail() {
		System.out.println("\n-->testGetAccount_Fail()");
		Person p = new Person("Nate");
		BasicAccount ba = new BasicAccount("123", 1000.0);
		GoldAccount ga1 = new GoldAccount("111", 500.0, 0.1);
		p.addAccount(ba);
		p.addAccount(ga1);
		BasicAccount a = p.getAccount(3);
		System.out.println("Account trying to retrieve:null");
		System.out.println("           p.getAccount(3):" + a);
	}

	private static void getAccountWithNumber_Success() {
		System.out.println("\n-->getAccountWithNumber_Success()");
		Person p = new Person("Nate");
		BasicAccount ba = new BasicAccount("123", 1000.0);
		GoldAccount ga1 = new GoldAccount("111", 500.0, 0.1);
		p.addAccount(ba);
		p.addAccount(ga1);
		BasicAccount a = p.getAccount("123");
		System.out.println("Account trying to retrieve:" + ba);
		System.out.println("       p.getAccount(\"123\"):" + a);
	}

	private static void getAccountWithNumber_Fail() {
		System.out.println("\n-->getAccountWithNumber_Fail()");
		Person p = new Person("Nate");
		BasicAccount ba = new BasicAccount("123", 1000.0);
		GoldAccount ga1 = new GoldAccount("111", 500.0, 0.1);
		p.addAccount(ba);
		p.addAccount(ga1);
		BasicAccount a = p.getAccount("555");
		System.out.println("Account trying to retrieve:null");
		System.out.println("       p.getAccount(\"555\"):" + a);
	}

	private static void getAccountsWithNumber_Success() {
		System.out.println("\n-->getAccountsWithNumber_Success()");
		Person p = buildPerson();
		System.out.println("Accounts");
		printPerson(p);
		System.out.println();

		ArrayList<BasicAccount> acnts = p.getAccountsWithNumber("12");
		System.out.println("Accounts returned from:p.getAccountsWithNumber(\"12\")");
		for(BasicAccount a : acnts) {
			System.out.println(a);
		}
	}

	private static void getAccountsWithNumber_Fail() {
		System.out.println("\n-->getAccountsWithNumber_Success_Fail()");
		Person p = buildPerson();
		System.out.println("Accounts");
		printPerson(p);
		System.out.println();

		ArrayList<BasicAccount> acnts = p.getAccountsWithNumber("9");
		System.out.println("Accounts returned from:p.getAccountsWithNumber(\"9\")");
		if(acnts.size()==0) {
			System.out.println("None");
		}
		else {
			for(BasicAccount a : acnts) {
				System.out.println(a);
			}
		}
	}

	private static void removeAccount_Success() {
		System.out.println("\n-->removeAccount_Success()");
		Person p = buildPerson();
		System.out.println("Accounts Before Remove");
		printPerson(p);
		System.out.println();
		
		BasicAccount a =p.removeAccount(1);

		System.out.println("Accounts After Remove: p.removeAccount(1)");
		printPerson(p);
		System.out.println();
		
		System.out.println("Account Removed:" + a);
	}

	private static void removeAccount_Fail() {
		System.out.println("\n-->removeAccount_Fail()");
		Person p = buildPerson();
		System.out.println("Accounts Before Remove");
		printPerson(p);
		System.out.println();
		
		BasicAccount a =p.removeAccount(4);

		System.out.println("Accounts After Remove: p.removeAccount(4)");
		printPerson(p);
		System.out.println();
		
		System.out.println("Accounts Removed:" + a);
	}

	private static void removeAccountWithNumber_Success() {
		System.out.println("\n-->removeAccountWithNumber_Success()");
		Person p = buildPerson();
		System.out.println("Accounts Before Remove");
		printPerson(p);
		System.out.println();
		
		BasicAccount a =p.removeAccount("111");

		System.out.println("Accounts After Remove: p.removeAccount(\"111\")");
		printPerson(p);
		System.out.println();
		
		System.out.println("Account Removed:" + a);
	}

	private static void removeAccountWithNumber_Fail() {
		System.out.println("\n-->removeAccountWithNumber_Fail()");
		Person p = buildPerson();
		System.out.println("Accounts Before Remove");
		printPerson(p);
		System.out.println();
		
		BasicAccount a =p.removeAccount("9");

		System.out.println("Accounts After Remove: p.removeAccount(\"9\")");
		printPerson(p);
		System.out.println();
		
		System.out.println("Account Removed:" + a);
	}

	private static void hasAccount_Success() {
		System.out.println("\n-->hasAccount_Success()");
		Person p = buildPerson();
		System.out.println("Accounts");
		printPerson(p);
		System.out.println();
		
		boolean hasAccount = p.hasAccount("127");

		System.out.println("p.hasAccount(\"127\")=" + hasAccount);
	}

	private static void hasAccount_Fail() {
		System.out.println("\n-->hasAccount_Fail()");
		Person p = buildPerson();
		System.out.println("Accounts");
		printPerson(p);
		System.out.println();
		
		boolean hasAccount = p.hasAccount("999");

		System.out.println("p.hasAccount(\"999\")=" + hasAccount);
	}

	private static void testApplyInterest() {
		System.out.println("\ntestApplyInterest()");
		Person p = buildPerson();
		System.out.println("Accounts before interest");
		printPerson(p);
		System.out.println();
		
		p.applyInterest();

		System.out.println("Accounts interest interest");
		printPerson(p);
	}

	private static void testGetTotalBalance() {
		System.out.println("\ntestGetTotalBalance()");
		Person p = buildPerson();
		System.out.println("Accounts");
		printPerson(p);
		double totalBalance = p.getTotalBalance();
		System.out.println("Total balance=" + totalBalance);
	}

	private static void testGetTotalGoldAccounts() {
		System.out.println("\ntestGetTotalGoldAccounts()");
		Person p = buildPerson();
		System.out.println("Accounts");
		printPerson(p);
		double totalBalance = p.getTotalGoldAccounts();
		System.out.println("Total balance Gold Accounts=" + totalBalance);
	}

	private static void testGetSmallestInterestRate() {
		System.out.println("\ntestGetSmallestInterestRate()");
		Person p = buildPerson();
		System.out.println("Accounts");
		printPerson(p);
		System.out.println();
		
		double smallestInterest = p.getSmallestInterestRate();
		System.out.println("Smallest Interest Rate=" + smallestInterest);
	}

	private static void testGetGoldAccounts() {
		System.out.println("\ntestGetGoldAccounts()");
		Person p = buildPerson();
		System.out.println("Accounts");
		printPerson(p);
		System.out.println();

		ArrayList<GoldAccount> golds = p.getGoldAccounts();

		System.out.println("Gold Accounts");
		for(GoldAccount ga : golds) {
			System.out.println(ga);
		}
	}

	private static Person buildPerson() {
		Person p = new Person("Nate");
		BasicAccount ba1 = new BasicAccount("123", 1000.0);
		GoldAccount ga1 = new GoldAccount("111", 500.0, 0.1);
		BasicAccount ba2= new BasicAccount("127", 1000.0);
		GoldAccount ga2 = new GoldAccount("011", 1500.0, 0.05);
		p.addAccount(ba1); 
		p.addAccount(ga1);
		p.addAccount(ba2); 
		p.addAccount(ga2);
		
		return p;
	}
	
	private static void printPerson(Person p) {
		for(int i=0; i<p.getNumAccounts(); i++) {
			System.out.println(p.getAccount(i));
		}
	}
}
